import time
from .XCP_Connection_Fixture import FixtureXCPcommunication, xcp_controller_instance
from xtr import logging

# Set up a logger for this module.
logger = logging.getLogger(__name__)


# Create a test case class that inherits from the preset TestFixture.
class TestXCPCommunication(FixtureXCPcommunication):
    def setUp(self):

        logger.info("Setup test case")

    def tearDown(self):

        logger.info("Tear down test case")

    def test_write_read_xcp(self):
        """
        Test the write and read functionality of XCP communication.

        This test initializes the XCP controller, connects to the ECU,
        writes a value to an A2L object, reads it back, and verifies
        that the written value matches the read value.
        """
        # Define parameters for the test
        ECU_NAME = "Ecu Name"
        CHANNEL_NAME = "Channel Name"
        A2L_FILE_PATH = "A2L file path"
        protocol = "Transport protocol"
        dst_host = "dst ip add"
        dst_port = "dst port"
        a2l_object_to_write = "xcp_object_name"
        value_to_write = "value to write"
        a2l_object_to_read = "xcp_object_name"  # This should match the object being written

        # Initialize and connect the XCP controller
        xcp_controller_instance.init(ECU_NAME, CHANNEL_NAME, A2L_FILE_PATH)
        connect_xcp = xcp_controller_instance.connect(
            ecu_name=ECU_NAME,
            channel_name=CHANNEL_NAME,
            protocol=protocol,
            dst_host=dst_host,
            dst_port=dst_port
        )

        # Check the connection status
        if connect_xcp:
            logger.info("Connection state OK for ECU {}".format(ECU_NAME))
        else:
            logger.error("Connection state NOT OK for ECU {}".format(ECU_NAME))
            return  # Exit the test if the connection fails

        # Write the specified value to the A2L object
        write_xcp_object = xcp_controller_instance.write(
            ecu_name=ECU_NAME,
            channel_name=CHANNEL_NAME,
            a2l_object=a2l_object_to_write,
            value=value_to_write
        )

        # Check the result of the write operation
        if write_xcp_object:
            logger.info("XCP-Write OK")
        else:
            logger.error("XCP-Write ERROR")
            return  # Exit the test if the write fails

        # Wait for a moment to ensure the write operation is completed
        time.sleep(1)

        # Read the value back from the A2L object
        return_value = xcp_controller_instance.read_XCP(CHANNEL_NAME, a2l_object_to_read, ECU_NAME)

        # Verify that the written value matches the read value
        assert value_to_write == return_value, "The A2L object value written is different from the returned value."

        # Wait for a moment before disconnecting
        time.sleep(1)

        # Disconnect the XCP controller
        xcp_controller_instance.disconnect()
